@@ -2,7 +2,7 @@
 /*
 
 cellwriter -- a character recognition input method
-Copyright (C) 2007 Michael Levin <risujin@risujin.org>
+Copyright (C) 2007 Michael Levin <risujin@gmail.com>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -57,7 +57,7 @@ static void stop_drawing(void);
 
 struct Cell {
         Sample sample, *alts[ALTERNATES];
-        gunichar2 ch;
+        gunichar ch;
         int alt_used[ALTERNATES];
         char flags, alt_ratings[ALTERNATES];
 };
@@ -86,7 +86,7 @@ static cairo_t *cairo = NULL;
 static PangoContext *pango = NULL;
 static PangoFontDescription *pango_font_desc = NULL;
 static KeyWidget *key_widget;
-static gunichar2 *history[HISTORY_MAX];
+static gunichar *history[HISTORY_MAX];
 static int cell_cols, cell_rows, cell_row_view = 0, current_cell = -1, old_cc,
            cell_cols_saved, cell_rows_saved, cell_row_view_saved,
            timeout_source,
@@ -1069,7 +1069,6 @@ static void unclear(int render)
 
 static void draw(double x, double y)
 {
-        Stroke *s;
         int cx, cy;
 
         if (current_cell < 0)
@@ -1087,11 +1086,10 @@ static void draw(double x, double y)
         }
 
         /* Allocate a new stroke if we aren't already drawing */
-        s = input->strokes[input->len - 1];
         if (!drawing) {
                 if (input->len >= STROKES_MAX)
                         return;
-                s = input->strokes[input->len++]= stroke_new(0);
+                input->strokes[input->len++]= stroke_new(0);
                 drawing = TRUE;
                 if (input->len == 1)
                         render_cell(current_cell);
@@ -1475,13 +1473,9 @@ static gboolean motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
 #endif
 
         /* If we are getting invalid output from this device with XInput
-           enabled, try disabling it */
-        if ((x < 0 || x > drawing_area->allocation.width ||
-             y < 0 || y > drawing_area->allocation.width) &&
-            event->device->mode != GDK_MODE_DISABLED && xinput_enabled) {
-                g_warning("Extended input device is generating invalid "
-                          "coordinates, disabled");
-                gdk_device_set_mode(event->device, GDK_MODE_DISABLED);
+           enabled, ignore it */
+        if (x < 0 || x > drawing_area->allocation.width ||
+            y < 0 || y > drawing_area->allocation.width) {
                 return TRUE;
         }
 
@@ -1644,28 +1638,6 @@ static gboolean enter_notify_event(GtkWidget *widget, GdkEventCrossing *event)
         return FALSE;
 }
 
-static gboolean leave_notify_event(GtkWidget *widget, GdkEventCrossing *event)
-{
-        /* Tablet PC gets grab leave-notify event when starting to draw.
-           Ignore this if we are still drawing. */
-        if (event->mode == GDK_CROSSING_GRAB || drawing || cross_out)
-                return FALSE;
-
-        old_cc = current_cell;
-        current_cell = -1;
-        finish_cell(old_cc);
-        if (inserting) {
-                inserting = FALSE;
-                dirty_cell(old_cc);
-                dirty_cell(old_cc - 1);
-        }
-        invalid = TRUE;
-        cell_widget_set_cursor(FALSE);
-        render_dirty();
-        start_timeout();
-        return FALSE;
-}
-
 static void scrollbar_value_changed(void)
 /* The cell widget has been scrolled */
 {
@@ -1755,8 +1727,6 @@ const char *cell_widget_word(void)
 
 void cell_widget_clear(void)
 {
-        int resized;
-
         stop_timeout();
         free_cells();
 
@@ -1767,7 +1737,7 @@ void cell_widget_clear(void)
                 cell_cols = cell_cols_saved;
                 cell_row_view = cell_row_view_saved;
                 training = FALSE;
-                resized = pack_cells(cell_rows, cell_cols);
+                pack_cells(cell_rows, cell_cols);
 
                 /* Show the on-screen keyboard */
                 if (check_clear()) {
@@ -1778,16 +1748,14 @@ void cell_widget_clear(void)
 
         /* Clear cells otherwise */
         else {
-                resized = pack_cells(1, cell_cols);
+                pack_cells(1, cell_cols);
 
                 /* Show the on-screen keyboard */
                 show_keys = keyboard_enabled;
                 is_clear = TRUE;
         }
 
-        /* Only re-render when we aren't going to get a configure event */
-        if (!resized)
-                cell_widget_render();
+        cell_widget_render();
 }
 
 void cell_widget_train(void)
@@ -1823,7 +1791,7 @@ void cell_widget_train(void)
 
         /* Preset all of the characters for training */
         for (i = 0, pos = 0; i < range; i++) {
-                short ch;
+                gunichar ch;
 
                 ch = block->start + i;
                 if (char_disabled(ch))
@@ -1873,7 +1841,7 @@ void cell_widget_pack(void)
 
 int cell_widget_insert(void)
 {
-        gunichar2 *utf16;
+        gunichar *utf16;
         int i, j, slot, chars;
 
         if (training)
@@ -1939,7 +1907,7 @@ static void buffer_menu_deactivate(GtkMenuShell *shell, GtkWidget *button)
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
 }
 
-static void buffer_menu_item_activate(GtkWidget *widget, gunichar2 *history)
+static void buffer_menu_item_activate(GtkWidget *widget, gunichar *history)
 {
         int i;
 
@@ -1987,9 +1955,9 @@ void cell_widget_show_buffer(GtkWidget *button)
                 gchar *string;
 
                 /* Convert string from a UTF-16 array to displayable UTF-8 */
-                string = g_utf16_to_utf8(history[i], -1, NULL, NULL, &error);
+                string = g_ucs4_to_utf8(history[i], -1, NULL, NULL, &error);
                 if (error) {
-                        g_warning("g_utf16_to_utf8(): %s", error->message);
+                        g_warning("g_ucs4_to_utf8(): %s", error->message);
                         continue;
                 }
 
@@ -2042,7 +2010,7 @@ int cell_widget_get_height(void)
 GtkWidget *cell_widget_new(void)
 /* Creates the Cell widget. Should only be called once per program run! */
 {
-        PangoCairoFontMap *font_map;
+        PangoFontMap *font_map;
         GtkWidget *widget, *hbox;
 
         /* Initial settings */
@@ -2064,12 +2032,12 @@ GtkWidget *cell_widget_new(void)
                          G_CALLBACK(motion_notify_event), NULL);
         g_signal_connect(G_OBJECT(drawing_area), "enter_notify_event",
                          G_CALLBACK(enter_notify_event), NULL);
-        g_signal_connect(G_OBJECT(drawing_area), "leave_notify_event",
-                         G_CALLBACK(leave_notify_event), NULL);
         g_signal_connect(G_OBJECT(drawing_area), "scroll_event",
                          G_CALLBACK(scroll_event), NULL);
         g_signal_connect(G_OBJECT(drawing_area), "style-set",
                          G_CALLBACK(cell_widget_update_colors), NULL);
+        // Do not listen to leave_notify_event, after a certain GTK version
+        // it fires for just about anything you do on the screen.
         gtk_widget_set_events(drawing_area,
                               GDK_EXPOSURE_MASK |
                               GDK_BUTTON_PRESS_MASK |
@@ -2077,7 +2045,6 @@ GtkWidget *cell_widget_new(void)
                               GDK_POINTER_MOTION_MASK |
                               GDK_POINTER_MOTION_HINT_MASK |
                               GDK_ENTER_NOTIFY_MASK |
-                              GDK_LEAVE_NOTIFY_MASK |
                               GDK_SCROLL_MASK);
 
         /* Update colors */
@@ -2114,8 +2081,8 @@ GtkWidget *cell_widget_new(void)
         pango_font_description_set_family(pango_font_desc, "Monospace");
 
         /* Pango context */
-        font_map = PANGO_CAIRO_FONT_MAP(pango_cairo_font_map_new());
-        pango = pango_cairo_font_map_create_context(font_map);
+        font_map = pango_cairo_font_map_new();
+        pango = pango_font_map_create_context(font_map);
         g_object_unref(font_map);
 
         /* Clear cells */
@@ -2145,4 +2112,3 @@ void cell_widget_cleanup(void)
         if (pango)
                 g_object_unref(pango);
 }
-
