@@ -2,7 +2,7 @@
 /*
 
 cellwriter -- a character recognition input method
-Copyright (C) 2007 Michael Levin <risujin@risujin.org>
+Copyright (C) 2007 Michael Levin <risujin@gmail.com>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -36,6 +36,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 /* Define this to print key events without actually generating them */
 /* #define DEBUG_KEY_EVENTS */
 
+/* Note about libfakekey: this library does very much the same thing as this
+   code and is now packaged in Ubuntu. However, it is hardcoded to "recycle"
+   only 10 keycodes rather than cycling through all unused keys as this code
+   does. This is sufficient only for periodic single key-presses. If a user
+   handwrites a long paragraph of Unicode characters, we want to be able to
+   accomodate as many as we can. */
+
 /*
         X11 KeyCodes
 */
@@ -104,15 +111,19 @@ static void release_keycode(KeyCode k)
 static void press_keycode(KeyCode k)
 /* Called from various places to generate a key-down event */
 {
-        if (k >= key_min && k <= key_max)
+        if (k >= key_min && k <= key_max) {
                 XTestFakeKeyEvent(GDK_DISPLAY(), k, True, 1);
+                XSync(GDK_DISPLAY(), False);
+        }
 }
 
 static void release_keycode(KeyCode k)
 /* Called from various places to generate a key-up event */
 {
-        if (k >= key_min && k <= key_max)
+        if (k >= key_min && k <= key_max) {
                 XTestFakeKeyEvent(GDK_DISPLAY(), k, False, 1);
+                XSync(GDK_DISPLAY(), False);
+        }
 }
 
 #endif
@@ -235,20 +246,6 @@ static void reset_keyboard(void)
                 type_keycode(ke_num_lock.keycode);
 }
 
-static int is_modifier(unsigned int keysym)
-/* Returns TRUE for KeySyms that are tracked internally */
-{
-        switch (keysym) {
-        case XK_Shift_L:
-        case XK_Shift_R:
-        case XK_Num_Lock:
-        case XK_Caps_Lock:
-                return TRUE;
-        default:
-                return FALSE;
-        }
-}
-
 static void key_event_allocate(KeyEvent *key_event, unsigned int keysym)
 /* Either finds the KeyCode associated with the given keysym or overwrites
    a usable one to generate it */
@@ -324,11 +321,9 @@ static void key_event_allocate(KeyEvent *key_event, unsigned int keysym)
 }
 
 void key_event_new(KeyEvent *key_event, unsigned int keysym)
-/* Filters locks and shifts but allocates other keys normally */
+/* Allocates key event */
 {
         key_event->keysym = keysym;
-        if (is_modifier(keysym))
-                return;
         key_event_allocate(key_event, keysym);
 }
 
@@ -342,22 +337,9 @@ void key_event_free(KeyEvent *key_event)
         key_event->keysym = 0;
 }
 
-void key_event_press(KeyEvent *key_event)
-/* Press the KeyCode specified in the event */
+void key_event_press_force(KeyEvent *key_event)
+/* Press the KeyCode specified in the event without sticky key tracking */
 {
-        /* Track modifiers without actually using them */
-        if (key_event->keysym == XK_Shift_L ||
-            key_event->keysym == XK_Shift_R) {
-                key_shifted++;
-                return;
-        } else if (key_event->keysym == XK_Caps_Lock) {
-                key_caps_locked = !key_caps_locked;
-                return;
-        } else if (key_event->keysym == XK_Num_Lock) {
-                key_num_locked = !key_num_locked;
-                return;
-        }
-
         /* Invalid event */
         if (key_event->keycode < key_min || key_event->keycode > key_max)
                 return;
@@ -378,15 +360,28 @@ void key_event_press(KeyEvent *key_event)
         XSync(GDK_DISPLAY(), False);
 }
 
-void key_event_release(KeyEvent *key_event)
+void key_event_press(KeyEvent *key_event)
+/* Press the KeyCode specified in the event */
 {
         /* Track modifiers without actually using them */
         if (key_event->keysym == XK_Shift_L ||
             key_event->keysym == XK_Shift_R) {
-                key_shifted--;
+                key_shifted++;
+                return;
+        } else if (key_event->keysym == XK_Caps_Lock) {
+                key_caps_locked = !key_caps_locked;
+                return;
+        } else if (key_event->keysym == XK_Num_Lock) {
+                key_num_locked = !key_num_locked;
                 return;
         }
 
+        key_event_press_force(key_event);
+}
+
+void key_event_release_force(KeyEvent *key_event)
+/* Press the KeyCode specified in the event without sticky key tracking */
+{
         /* Invalid key event */
         if (key_event->keycode < key_min || key_event->keycode > key_max)
                 return;
@@ -401,6 +396,19 @@ void key_event_release(KeyEvent *key_event)
         XSync(GDK_DISPLAY(), False);
 }
 
+void key_event_release(KeyEvent *key_event)
+/* Release the KeyCode specified in the event */
+{
+        /* Track modifiers without actually using them */
+        if (key_event->keysym == XK_Shift_L ||
+            key_event->keysym == XK_Shift_R) {
+                key_shifted--;
+                return;
+        }
+
+        key_event_release_force(key_event);
+}
+
 #ifdef X_HAVE_UTF8_STRING
 void key_event_send_char(int unichar)
 {
@@ -520,4 +528,3 @@ void key_event_cleanup(void)
 {
         cleanup_usable();
 }
-
