@@ -2,7 +2,7 @@
 /*
 
 cellwriter -- a character recognition input method
-Copyright (C) 2007 Michael Levin <risujin@risujin.org>
+Copyright (C) 2007 Michael Levin <risujin@gmail.com>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,7 +34,9 @@ extern int key_shifted, key_num_locked, key_caps_locked;
 void key_event_new(KeyEvent *key_event, unsigned int keysym);
 void key_event_free(KeyEvent *key_event);
 void key_event_press(KeyEvent *key_event);
+void key_event_press_force(KeyEvent *key_event);
 void key_event_release(KeyEvent *key_event);
+void key_event_release_force(KeyEvent *key_event);
 void key_event_send_char(int unichar);
 void key_event_send_enter(void);
 void key_event_update_mappings(void);
@@ -95,4 +97,3 @@ void key_widget_configure(KeyWidget *key_widget, int x, int y,
 /* Functions to update keyboards */
 int key_widget_update_colors(void);
 void key_widget_cleanup(KeyWidget *key_widget);
-
